(
	function()
	{
		"use strict"

		class Overlay
		{
			/*
			   ###     ######   ######  ########  ######   ######   #######  ########   ######
			  ## ##   ##    ## ##    ## ##       ##    ## ##    ## ##     ## ##     ## ##    ##
			 ##   ##  ##       ##       ##       ##       ##       ##     ## ##     ## ##
			##     ## ##       ##       ######    ######   ######  ##     ## ########   ######
			######### ##       ##       ##             ##       ## ##     ## ##   ##         ##
			##     ## ##    ## ##    ## ##       ##    ## ##    ## ##     ## ##    ##  ##    ##
			##     ##  ######   ######  ########  ######   ######   #######  ##     ##  ######
			*/
			// <editor-fold desc="accessors">
			/*
			 ######  ########    ###    ######## ####  ######
			##          ##     ##   ##     ##     ##  ##
			 ######     ##    ##     ##    ##     ##  ##
			      ##    ##    #########    ##     ##  ##
			 ######     ##    ##     ##    ##    ####  ######
			*/
			// <editor-fold desc="static">
			static get settings()
			{
				return Object.freeze
				({
					content:
					{
						height:	"auto",
						width:	"auto",
					},
					shadow: true,
				})
			}
			// </editor-fold>
			/*
			########  ########  #### ##     ##    ###    ######## ########
			##     ## ##     ##  ##  ##     ##  ##   ##     ##    ##
			########  ########   ##  ##     ## ##     ##    ##    ######
			##        ##   ##    ##   ##   ##  #########    ##    ##
			##        ##     ## ####    ###    ##     ##    ##    ########
			*/
			// <editor-fold desc="private">
			set _element(eElement)
			{
				this.oElements.eElement = eElement
			}
			// </editor-fold>
			/*
			########  ##     ## ########  ##       ####  ######
			##     ## ##     ## ##     ## ##        ##  ##
			########  ##     ## ########  ##        ##  ##
			##        ##     ## ##     ## ##        ##  ##
			##         #######  ########  ######## ####  ######
			*/
			// <editor-fold desc="public">
			get content()
			{
				return this.vContent
			}
			set content(vContent)
			{
				this.vContent = vContent
			}
			get element()
			{
				return this.oElements.eElement
			}
			get settings()
			{
				return this.oSettings
			}
			set settings(oSettings)
			{
				this.oSettings = $.extend(true, {}, Overlay.settings, this.settings || {}, oSettings)
			}
			get target()
			{
				return this.oElements.eTarget
			}
			set target(eTarget)
			{
				this.oElements.eTarget = eTarget
			}
			// </editor-fold>
			// </editor-fold>
			/*
			######## ##     ## ######## ##    ## ########  ######
			##       ##     ## ##       ###   ##    ##    ##    ##
			##       ##     ## ##       ####  ##    ##    ##
			######   ##     ## ######   ## ## ##    ##     ######
			##        ##   ##  ##       ##  ####    ##          ##
			##         ## ##   ##       ##   ###    ##    ##    ##
			########    ###    ######## ##    ##    ##     ######
			*/
			// <editor-fold desc="events">
			onResize()
			{
				if(this.isCreated())
				{
					$(this.element).css(this._targetSize())
				}
			}
			onUnload()
			{
				this.destructor()
			}
			// </editor-fold>
			/*
			 ######  ######## ########  ##     ##  ######  ########  #######  ########   ######
			##    ##    ##    ##     ## ##     ## ##    ##    ##    ##     ## ##     ## ##    ##
			##          ##    ##     ## ##     ## ##          ##    ##     ## ##     ## ##
			 ######     ##    ########  ##     ## ##          ##    ##     ## ########   ######
			      ##    ##    ##   ##   ##     ## ##          ##    ##     ## ##   ##         ##
			##    ##    ##    ##    ##  ##     ## ##    ##    ##    ##     ## ##    ##  ##    ##
			 ######     ##    ##     ##  #######   ######     ##     #######  ##     ##  ######
			*/
			// <editor-fold desc="structors">
			constructor(eTarget, vContent, oSettings)
			{
				this.vContent	= null
				this.oSettings = null
				this.oElements	=
				{
					eElement:	null,
					eTarget:	null,
				}


				this.content	= vContent	|| null
				this.settings	= oSettings || {}
				this.target	= eTarget	|| null

				this.onResize	= this.onResize	.bind(this)
				this.onUnload	= this.onUnload	.bind(this)

				window.addEventListener("beforeunload"	, this.onUnload)
				window.addEventListener("resize"	, this.onResize)

				if(this.hasContent() && this.hasTarget())
				{
					this.create()
				}

			}
			destructor()
			{
				this.destroy()

				this.target = null

				window.removeEventListener("beforeunload"	, this.onUnload)
				window.removeEventListener("resize"	, this.onResize)

				return this
			}
			// </editor-fold>
			/*
			##     ## ######## ######## ##     ##  #######  ########   ######
			###   ### ##          ##    ##     ## ##     ## ##     ## ##    ##
			#### #### ##          ##    ##     ## ##     ## ##     ## ##
			## ### ## ######      ##    ######### ##     ## ##     ##  ######
			##     ## ##          ##    ##     ## ##     ## ##     ##       ##
			##     ## ##          ##    ##     ## ##     ## ##     ## ##    ##
			##     ## ########    ##    ##     ##  #######  ########   ######
			*/
			// <editor-fold desc="methods">
			/*
			########  ########  #### ##     ##    ###    ######## ########
			##     ## ##     ##  ##  ##     ##  ##   ##     ##    ##
			########  ########   ##  ##     ## ##     ##    ##    ######
			##        ##   ##    ##   ##   ##  #########    ##    ##
			##        ##     ## ####    ###    ##     ##    ##    ########
			 */
			// <editor-fold desc="private">
			_targetSize(eTarget)
			{
				var	oCss

				eTarget = eTarget || this.target

				if(document.body.isSameNode(eTarget))
				{
					oCss =
					{
						height:	"100%"	,
						left:	0	,
						position:	"fixed"	,
						top:	0	,
						width:	"100%"	,
					}
				}
				else
				{
					oCss =
					{
						height:	window.parseFloat($(eTarget).outerHeight(true)	)	,
						left:	window.parseFloat(eTarget.offsetLeft	)	,
						top:	window.parseFloat(eTarget.offsetTop	)	,
						width:	window.parseFloat($(eTarget).outerWidth(true)	)	,
					}
				}

				return oCss
			}
			// </editor-fold>
			/*
			########  ##     ## ########  ##       ####  ######
			##     ## ##     ## ##     ## ##        ##  ##
			########  ##     ## ########  ##        ##  ##
			##        ##     ## ##     ## ##        ##  ##
			##         #######  ########  ######## ####  ######
			 */
			// <editor-fold desc="public">
			create(eTarget, vContent, oSettings)
			{
				var	oDocumentFragment

				vContent	= vContent	|| this.content
				oSettings	= oSettings	|| this.settings
				eTarget	= eTarget	|| this.target

				oDocumentFragment	= document.createDocumentFragment()

				if(eTarget !== null && vContent !== null)
				{
					if(this.isCreated())
					{
						this.destroy()
					}
					$(oDocumentFragment).append
					(
						$("<div />")
						.addClass("overlay")
						.css(this._targetSize(eTarget))
						.append(oSettings.shadow ? $("<div />").addClass("overlay-shadow") : undefined)
						.append
						(
							$("<div />")
							.addClass("overlay-content")
							.css
							(
								{
									height:	oSettings.content.height	,
									width:	oSettings.content.width	,
								}
							)
							.html(vContent)
						)
					)

					this._element = oDocumentFragment.children[0]
					if(document.body.isSameNode(eTarget))
					{
						eTarget.appendChild(oDocumentFragment)
					}
					else
					{
						eTarget.parentElement.insertBefore(oDocumentFragment, eTarget.nextElementSibling)
					}
				}

				return this
			}
			destroy()
			{
				if(this.isCreated())
				{
					this.element.remove()
					this._element = null
				}

				return this
			}
			hasContent()
			{
				var bReturnValue

				if(this.isCreated())
				{
					bReturnValue = this.element.children.length > 0
				}
				else
				{
					bReturnValue = this.content !== null
				}

				return bReturnValue
			}
			hasTarget()
			{
				return this.target !== null
			}
			isCreated()
			{
				return this.element !== null
			}
			// </editor-fold>
			// </editor-fold>
		}

		if(window.metasec === undefined)
		{
			window.metasec = {}
		}
		window.metasec.Overlay = Overlay
	}()
)
