(
	function()
	{
		"use strict"

		var oQrCodeReader

		oQrCodeReader =
		{
			/*
			   ###    ######## ######## ########  #### ########  ##     ## ######## ########  ######
			  ## ##      ##       ##    ##     ##  ##  ##     ## ##     ##    ##    ##       ##    ##
			 ##   ##     ##       ##    ##     ##  ##  ##     ## ##     ##    ##    ##       ##
			##     ##    ##       ##    ########   ##  ########  ##     ##    ##    ######    ######
			#########    ##       ##    ##   ##    ##  ##     ## ##     ##    ##    ##             ##
			##     ##    ##       ##    ##    ##   ##  ##     ## ##     ##    ##    ##       ##    ##
			##     ##    ##       ##    ##     ## #### ########   #######     ##    ########  ######
			*/
			// <editor-fold desc="attributes">
			nUploadTimer:	null,

			oOverlay:	null,
			oQCodeDecoder:	null,

			oElements:
			{
				video:	null,
			},
			// </editor-fold>
			/*
			   ###     ######   ######  ########  ######   ######   #######  ########   ######
			  ## ##   ##    ## ##    ## ##       ##    ## ##    ## ##     ## ##     ## ##    ##
			 ##   ##  ##       ##       ##       ##       ##       ##     ## ##     ## ##
			##     ## ##       ##       ######    ######   ######  ##     ## ########   ######
			######### ##       ##       ##             ##       ## ##     ## ##   ##         ##
			##     ## ##    ## ##    ## ##       ##    ## ##    ## ##     ## ##    ##  ##    ##
			##     ##  ######   ######  ########  ######   ######   #######  ##     ##  ######
			*/
			// <editor-fold desc="accessors">
			get elements()
			{
				return this.oElements
			},
			get overlay()
			{
				return this.oOverlay
			},
			set overlay(oOverlay)
			{
				this.oOverlay = oOverlay
			},
			get qCodeDecoder()
			{
				return this.oQCodeDecoder
			},
			set qCodeDecoder(oQCodeDecoder)
			{
				this.oQCodeDecoder = oQCodeDecoder
			},
			get timer()
			{
				return this.nUploadTimer
			},
			set timer(nTimer)
			{
				if(this.timer !== null)
				{
					window.clearTimeout(this.timer)
				}

				this.nUploadTimer = nTimer
			},
			// </editor-fold>
			/*
			######## ##     ## ######## ##    ## ########  ######
			##       ##     ## ##       ###   ##    ##    ##    ##
			##       ##     ## ##       ####  ##    ##    ##
			######   ##     ## ######   ## ## ##    ##     ######
			##        ##   ##  ##       ##  ####    ##          ##
			##         ## ##   ##       ##   ###    ##    ##    ##
			########    ###    ######## ##    ##    ##     ######
			*/
			// <editor-fold desc="events">
			onQrCodeResult(sResult)
			{
				this.upload(sResult)
				.then
				(
					() =>
					{
						this.timer = null
						this.stop()
					}
				)
				.catch
				(
					() =>
					{
						this.timer = window.setTimeout(() => {this.onQrCodeResult(sResult)}, 2500)
					}
				)
			},
			onUnload()
			{
				this.finalize()
			},
			onVideoEnded()
			{
				this.stop()
			},
			onVideoStarted()
			{
				this.overlay.create()
			},
			// </editor-fold>
			/*
			 ######  ######## ########  ##     ##  ######  ########  #######  ########   ######
			##    ##    ##    ##     ## ##     ## ##    ##    ##    ##     ## ##     ## ##    ##
			##          ##    ##     ## ##     ## ##          ##    ##     ## ##     ## ##
			 ######     ##    ########  ##     ## ##          ##    ##     ## ########   ######
			      ##    ##    ##   ##   ##     ## ##          ##    ##     ## ##   ##         ##
			##    ##    ##    ##    ##  ##     ## ##    ##    ##    ##     ## ##    ##  ##    ##
			 ######     ##    ##     ##  #######   ######     ##     #######  ##     ##  ######
			*/
			// <editor-fold desc="structors">
			initialize()
			{
				var oCrosshairSvgDownload

				oCrosshairSvgDownload = new this._xhr({method: "GET", url: "/img/crosshair.svg",})

				this.elements.video	= document.querySelector("#qrcodescanner-video"	)

				this.onUnload	= this.onUnload	.bind(this)
				this.onVideoEnded	= this.onVideoEnded	.bind(this)
				this.onVideoStarted	= this.onVideoStarted	.bind(this)

				this.elements.video	.addEventListener("playing"	, this.onVideoStarted	)
				this.elements.video	.addEventListener("ended"	, this.onVideoEnded	)
				window	.addEventListener("beforeunload"	, this.onUnload	)

				this.overlay	= new window.metasec.Overlay(this.elements.video, null, {content: {height: "100%",}, shadow: false,})
				this.qCodeDecoder = new QCodeDecoder()

				oCrosshairSvgDownload
				.then
				(
					(sCrosshairSvg) =>
					{
						this.overlay.content = `<div id="qrcodescanner-overlay">${sCrosshairSvg}</div>`
					}
				)
				this.start()

				return this
			},
			finalize()
			{
				this.timer = null

				this.overlay.destructor()
				this.overlay	= undefined

				try
				{
					this.qCodeDecoder.stop()
				}
				catch(oError)
				{

				}
				this.qCodeDecoder	= undefined

				this.elements.video	.removeEventListener("playing"	, this.onVideoStarted	)
				this.elements.video	.removeEventListener("ended"	, this.onVideoEnded	)
				this.elements.video = undefined

				window	.removeEventListener("beforeunload"	, this.onUnload	)

				$("#qrcodescanner").parent().empty()

				return this
			},
			// </editor-fold>
			/*
			##     ## ######## ######## ##     ##  #######  ########   ######
			###   ### ##          ##    ##     ## ##     ## ##     ## ##    ##
			#### #### ##          ##    ##     ## ##     ## ##     ## ##
			## ### ## ######      ##    ######### ##     ## ##     ##  ######
			##     ## ##          ##    ##     ## ##     ## ##     ##       ##
			##     ## ##          ##    ##     ## ##     ## ##     ## ##    ##
			##     ## ########    ##    ##     ##  #######  ########   ######
			*/
			// <editor-fold desc="methods">
			/*
			########  ########  #### ##     ##    ###    ######## ########
			##     ## ##     ##  ##  ##     ##  ##   ##     ##    ##
			########  ########   ##  ##     ## ##     ##    ##    ######
			##        ##   ##    ##   ##   ##  #########    ##    ##
			##        ##     ## ####    ###    ##     ##    ##    ########
			*/
			// <editor-fold desc="private">
			_xhr: function(oData)
			{
				return new Promise
				(
					(fResolve, fReject) =>
					{
						var	oXhr

						oXhr = new XMLHttpRequest()
						oXhr.addEventListener
						(
							"load",
							() =>
							{
								if(oXhr.status >= 200 && oXhr.status < 300)
								{
									fResolve(oXhr.response)
								}
								else
								{
									fReject({status: oXhr.status, statusText: oXhr.statusText})
								}
							}
						)
						oXhr.addEventListener
						(
							"error",
							() =>
							{
								fReject({status: oXhr.status, statusText: oXhr.statusText})
							}
						)

						oXhr.open(oData.method, oData.url)
						oXhr.send(oData.data)
					}
				)
			},
			// </editor-fold>
			/*
			########  ##     ## ########  ##       ####  ######
			##     ## ##     ## ##     ## ##        ##  ##
			########  ##     ## ########  ##        ##  ##
			##        ##     ## ##     ## ##        ##  ##
			##         #######  ########  ######## ####  ######
			*/
			// <editor-fold desc="public">
			start()
			{
				this.qCodeDecoder.decodeFromCamera(this.elements.video, (oError, sResult) => {if(!oError){this.onQrCodeResult(sResult)}})

				return this
			},
			stop()
			{
				this.finalize()

				return this
			},
			upload(sQrCode)
			{
				return this._xhr({method: "GET", url: `/qrcode.php?qrcode=${sQrCode}`,})
			},
			// </editor-fold>
			// </editor-fold>
		}

		oQrCodeReader.initialize()
		//document.addEventListener("DOMContentLoaded", function(){oQrCodeReader.initialize()})
	}
)()
