(
	function()
	{
		"use strict";

		let	oTranslator

		oTranslator =
		{
			sApiKey:	"AIzaSyB71Zv7RJIPARRoiMHHVttP-02T_g24-CI"	,
			bListening:	false	,
			bSpeaking:	false	,
			sSourceLanguage:	null	,
			sTargetLanguage:	null	,

			oCurrentTranslation:	null	,
			oSpeechRecognition:	null	,

			oEventHandler:
			{
				afTranslationError:	[]	,
				afListened:	[]	,
				afRecognitionResult:	[]	,
				afRecognitionPartialResult:	[]	,
				afSourceLanguageChanged:	[]	,
				afSpoken:	[]	,
				afTargetLanguageChanged:	[]	,
				afTranslated:	[]	,
			},

			onParseError: function()
			{
				// TODO: Add Parse error handling

				return this
			},
			onSpeechRecognitionError: function()
			{
				// TODO: Add SpeechRecognition error handling

				return this
			},
			onSpeechRecognitionFailed: function()
			{
				// TODO: Add SpeechRecognition nomatch handling

				return this
			},
			onSpeechRecognitionResult: function(oResult)
			{
				let sResult

				sResult = oResult.results[oResult.results.length - 1][0].transcript

				if(oResult.results[oResult.results.length - 1].isFinal)
				{
					this._fireEvent("recognitionResult", [sResult])
					this.translate(sResult)
				}
				else
				{
					this._fireEvent("recognitionPartialResult", [sResult])
				}

				return this
			},
			onSpoken: function()
			{
				this.stopSpeaking()

				return this
			},
			onUnload: function()
			{
				this.finalize()

				return this
			},

			initialize: function()
			{
				this.oSpeechRecognition	= new (window.SpeechRecognition || window.webkitSpeechRecognition)
				this.oSpeechRecognition.continuous	= true
				this.oSpeechRecognition.interimResults	= true

				this.onSpeechRecognitionError	=	this.onSpeechRecognitionError	.bind(this)
				this.onSpeechRecognitionFailed	=	this.onSpeechRecognitionFailed	.bind(this)
				this.onSpeechRecognitionResult	=	this.onSpeechRecognitionResult	.bind(this)
				this.onSpoken	=	this.onSpoken	.bind(this)
				this.onUnload	=	this.onUnload	.bind(this)

				this.oSpeechRecognition	.addEventListener("error"	, this.onSpeechRecognitionError	)
				this.oSpeechRecognition	.addEventListener("nomatch"	, this.onSpeechRecognitionFailed	)
				this.oSpeechRecognition	.addEventListener("result"	, this.onSpeechRecognitionResult	)
				window	.addEventListener("beforeunload"	, this.onUnload	)

				window.speechSynthesis.getVoices()

				return this
			},
			finalize: function()
			{
				this.oSpeechRecognition	.removeEventListener("error"	, this.onSpeechRecognitionError	)
				this.oSpeechRecognition	.removeEventListener("nomatch"	, this.onSpeechRecognitionFailed	)
				this.oSpeechRecognition	.removeEventListener("result"	, this.onSpeechRecognitionResult	)
				window	.removeEventListener("beforeunload"	, this.onUnload	)

				this.oCurrentTranslation	= null
				this.oSpeechRecognition	= null

				return this
			},

			_eventName: function(sEvent)
			{
				if(typeof sEvent !== "string" || sEvent === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator._eventName\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected !\"\" found \"" + sEvent + "\""
					)
				}

				return "af" + sEvent[0].toUpperCase() + sEvent.slice(1)
			},
			_fireEvent: function(sEvent, avArguments = [])
			{
				sEvent = this._eventName(sEvent)

				if(typeof sEvent !== "string" || sEvent === "" || this.oEventHandler[sEvent] === undefined)
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator._fireEvent\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected [targetLanguageChanged] found \"" + sEvent + "\""
					)
				}

				this.oEventHandler[sEvent].forEach((fHandler) => {fHandler.apply(null, avArguments)})

				return this
			},
			_matchingVoice: function(sLanguage)
			{
				if(typeof sLanguage !== "string" || sLanguage === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator._getMatchingVoice\""
						+	"\nArgument: \"language\""
						+	"\nType: expected \"string\" found \"" + typeof sLanguage + "\""
						+	"\nValue: expected !\"\" found \"" + sLanguage + "\""
					)
				}

				return window.speechSynthesis.getVoices().find(oVoice => oVoice.name.includes("Google") && oVoice.lang.includes(sLanguage)) || this._matchingVoice(this._similarLanguage(sLanguage))
			},
			_similarLanguage: function(sLanguage)
			{
				var sSimilarLanguage

				switch(sLanguage)
				{
					/*
						Undecided: hu, no, sv, tr

						no, sv are close to each other and danish
					*/
					case "bg":
						sSimilarLanguage = "ru"
						break
					case "ro":
						sSimilarLanguage = "it"
						break
					default:
						sSimilarLanguage = "en"
						break
				}

				return sSimilarLanguage
			},

			addEventListener: function(sEvent, fHandler)
			{
				sEvent = this._eventName(sEvent)

				if(typeof sEvent !== "string" || sEvent === "" || this.oEventHandler[sEvent] === undefined)
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.addEventListener\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected [targetLanguageChanged] found \"" + sEvent + "\""
					)
				}
				if(typeof fHandler !== "function")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.addEventListener\""
						+	"\nArgument: \"handler\""
						+	"\nType: expected \"function\" found \"" + typeof fHandler + "\""
						+	"\nValue: expected function found\"" + fHandler + "\""
					)
				}

				this.oEventHandler[sEvent].push(fHandler)

				return this
			},
			isListening: function()
			{
				return this.bListening
			},
			isSpeaking: function()
			{
				return this.bSpeaking
			},
			listen: function(sLanguage = this.sourceLanguage())
			{
				if(typeof sLanguage !== "string" || sLanguage === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.listen\""
						+	"\nArgument: \"language\""
						+	"\nType: expected \"string\" found \"" + typeof sLanguage + "\""
						+	"\nValue: expected !\"\" found \"" + sLanguage + "\""
						+	"\nDefault: Translator.sourceLanguage()"
					)
				}

				this	.stopListening()
					.stopSpeaking()

				this.bListening = true
				this.oSpeechRecognition.lang = sLanguage

				this.oSpeechRecognition.start()

				return this
			},
			removeEventListener: function(sEvent, fHandler)
			{
				sEvent = this._eventName(sEvent)

				if(typeof sEvent !== "string" || sEvent === "" || this.oEventHandler[sEvent] === undefined)
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.addEventListener\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected [targetLanguageChanged] found \"" + sEvent + "\""
					)
				}
				if(typeof fHandler !== "function")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.addEventListener\""
						+	"\nArgument: \"handler\""
						+	"\nType: expected \"function\" found \"" + typeof fHandler + "\""
						+	"\nValue: expected function found\"" + fHandler + "\""
					)
				}

				this.oEventHandler[sEvent].splice(this.oEventHandler[sEvent].indexOf(fHandler), 1)

				return this
			},
			sourceLanguage: function(sNewLanguage)
			{
				var sOldLanguage

				if(sNewLanguage === undefined)
				{
					return this.sSourceLanguage
				}

				if((typeof sNewLanguage !== "string" && sNewLanguage !== null) || sNewLanguage === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.sourceLanguage\""
						+	"\nArgument: \"language\""
						+	"\nType: expected \"string|null\" found \"" + typeof sNewLanguage + "\""
						+	"\nValue: expected !\"\"|null found \"" + sNewLanguage + "\""
					)
				}

				sOldLanguage	=	this.sourceLanguage()
				this.sSourceLanguage	=	sNewLanguage

				this._fireEvent("sourceLanguageChanged", [sNewLanguage, sOldLanguage])

				return this
			},
			speak: function(sText, sLanguage = this.sourceLanguage())
			{
				let	oUtterance

				if(typeof sLanguage !== "string" || sLanguage === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.speak\""
						+	"\nArgument: \"language\""
						+	"\nType: expected \"string\" found \"" + typeof sLanguage + "\""
						+	"\nValue: expected !\"\" found \"" + sLanguage + "\""
					)
				}
				if(typeof sText !== "string" || sText === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.speak\""
						+	"\nArgument: \"text\""
						+	"\nType: expected \"string\" found \"" + typeof sText + "\""
						+	"\nValue: expected !\"\" found \"" + sText + "\""
					)
				}

				this	.stopListening()
					.stopSpeaking()

				this.bSpeaking	=	true
				oUtterance	= new SpeechSynthesisUtterance(sText)
				oUtterance.onend	= this.onSpoken
				oUtterance.voice	= this._matchingVoice(sLanguage)

				window.speechSynthesis.speak(oUtterance)

				return this
			},
			stopListening: function()
			{
				if(this.isListening())
				{
					this.bListening = false
					this.oSpeechRecognition.stop()
					this._fireEvent("listened")
				}

				return this
			},
			stopSpeaking: function()
			{
				if(this.isSpeaking())
				{
					this.bSpeaking = false
					window.speechSynthesis.cancel()
					this._fireEvent("spoken")
				}

				return this
			},
			targetLanguage: function(sNewLanguage)
			{
				var sOldLanguage

				if(sNewLanguage === undefined)
				{
					return this.sTargetLanguage
				}

				if((typeof sNewLanguage !== "string" && sNewLanguage !== null) || sNewLanguage === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.targetLanguage\""
						+	"\nArgument: \"language\""
						+	"\nType: expected \"string|null\" found \"" + typeof sNewLanguage + "\""
						+	"\nValue: expected !\"\"|null found \"" + sNewLanguage + "\""
					)
				}

				sOldLanguage	=	this.targetLanguage()
				this.sTargetLanguage	=	sNewLanguage

				this._fireEvent("targetLanguageChanged", [sNewLanguage, sOldLanguage])

				return this
			},
			translate: function(sText, sSource = this.sourceLanguage(), sTarget = this.targetLanguage())
			{
				let	oXhr

				if(typeof sText !== "string" || sText === "")
				{
					throw new TypeError
					(
						"incorrect type or unusable value"
						+	"\nMethod: \"Translator.translate\""
						+	"\nArgument: \"text\""
						+	"\nType: expected \"string\" found \"" + typeof sTarget + "\""
						+	"\nValue: expected !\"\" found \"" + sTarget + "\""
					)
				}
				if(typeof sSource !== "string" || sSource === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.translate\""
						+	"\nArgument: \"source\""
						+	"\nType: expected \"string\" found \"" + typeof sSource + "\""
						+	"\nValue: expected !\"\" found \"" + sSource + "\""
						+ "\nDefault: Translator.sourceLanguage()"
					)
				}
				if(typeof sTarget !== "string" || sTarget === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.translate\""
						+	"\nArgument: \"target\""
						+	"\nType: expected \"string\" found \"" + typeof sTarget + "\""
						+	"\nValue: expected !\"\" found \"" + sTarget + "\""
						+ "\nDefault: Translator.targetLanguage()"
					)
				}

				this.oCurrentTranslation	= {source: sSource, target: sTarget, text: sText}

				if(sSource !== sTarget)
				{
					oXhr	= new XMLHttpRequest()

					oXhr.addEventListener("load"	, () => {this._fireEvent("translated", [decodeURIComponent(JSON.parse(oXhr.responseText).data.translations[0].translatedText)])})
					oXhr.addEventListener("error"	, () => {this._fireEvent("translationError", oXhr.responseText)})
					oXhr.open("POST", "https://www.googleapis.com/language/translate/v2?key=" + this.sApiKey + "&source=" + window.encodeURIComponent(sSource) + "&target=" + window.encodeURIComponent(sTarget) + "&q=" + window.encodeURIComponent(sText))
					oXhr.send()
				}
				else
				{
					this._fireEvent("translated", [sText])
				}

				return this
			},
		}

		document.addEventListener
		(
			"DOMContentLoaded",
			function()
			{
				oTranslator.initialize()
			}
		)
		window.Translator = oTranslator
	}()
)
