(
	function()
	{
		var oConnector

		oConnector =
		{
			sUri:	`wss://${window.location.hostname}:33379/jlingo`	,

			oWebSocket:	null	,

			oEventHandler:
			{
				afAnswerTextReceived:	[]	,
				afConnected:	[]	,
				afError:	[]	,
				afDisconnected:	[]	,
			},

			onSourceLanguageChanged: function(sLanguage)
			{
				this.send("language")
			},
			onTranslated: function(sText)
			{
				if(!Ui.hasChangedLanguage())
				{
					this.send("message", sText)
				}

				return this
			},
			onUnload: function()
			{
				this.finalize()

				return this
			},
			onWebSocketClosed: function()
			{
				this._fireEvent("disconnected")
				window.setTimeout(() => {this._webSocket()}, 60000)

				return this
			},
			onWebSocketError: function()
			{
				this._fireEvent("error")
				// TODO: Add WebSocket error handling

				return this
			},
			onWebSocketMessage: function(sMessage)
			{
				var oData

				oData = JSON.parse(sMessage.data)

				switch(oData.type)
				{
					case "language":
						Translator.targetLanguage(oData.value)
						break
					case "message":
						this._fireEvent("answerTextReceived", [oData.value])
						break
				}

				return this
			},
			onWebSocketOpen: function()
			{
				this.send("language")
				this._fireEvent("connected")

				return this
			},

			initialize: function()
			{
				this.onSourceLanguageChanged	=	this.onSourceLanguageChanged	.bind(this)
				this.onTranslated	=	this.onTranslated	.bind(this)
				this.onUnload	=	this.onUnload	.bind(this)
				this.onWebSocketClosed	=	this.onWebSocketClosed	.bind(this)
				this.onWebSocketError	=	this.onWebSocketError	.bind(this)
				this.onWebSocketMessage	=	this.onWebSocketMessage	.bind(this)
				this.onWebSocketOpen	=	this.onWebSocketOpen	.bind(this)

				Translator	.addEventListener("sourceLanguageChanged"	,	this.onSourceLanguageChanged	)
				Translator	.addEventListener("translated"	,	this.onTranslated	)
				window	.addEventListener("beforeunload"	,	this.onUnload	)

				this._webSocket()

				return this
			},
			finalize: function()
			{
				this.oWebSocket	.removeEventListener("close"	,	this.onWebSocketClosed	)
				this.oWebSocket	.removeEventListener("error"	,	this.onWebSocketError	)
				this.oWebSocket	.removeEventListener("message"	,	this.onWebSocketMessage	)
				this.oWebSocket	.removeEventListener("open"	,	this.onWebSocketOpen	)
				Translator	.removeEventListener("sourceLanguageChanged"	,	this.onSourceLanguageChanged	)
				Translator	.removeEventListener("translated"	,	this.onTranslated	)
				window	.removeEventListener("beforeunload"	,	this.onUnload	)

				this.oWebSocket.close()

				this.oWebSocket = null

				return this
			},

			_eventName: function(sEvent)
			{
				if(typeof sEvent !== "string" || sEvent === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Connector._eventName\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected !\"\" found \"" + sEvent + "\""
					)
				}

				return "af" + sEvent[0].toUpperCase() + sEvent.slice(1)
			},
			_fireEvent: function(sEvent, avArguments = [])
			{
				sEvent = this._eventName(sEvent)

				if(typeof sEvent !== "string" || sEvent === "" || this.oEventHandler[sEvent] === undefined)
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Connector._fireEvent\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected [targetLanguageChanged] found \"" + sEvent + "\""
					)
				}

				this.oEventHandler[sEvent].forEach((fHandler) => {fHandler.apply(null, avArguments)})

				return this
			},
			_webSocket: function()
			{
				if(this.oWebSocket !== null)
				{
					this.oWebSocket	.removeEventListener("close"	, this.onWebSocketClosed	)
					this.oWebSocket	.removeEventListener("error"	, this.onWebSocketError	)
					this.oWebSocket	.removeEventListener("message"	, this.onWebSocketMessage	)
					this.oWebSocket	.removeEventListener("open"	, this.onWebSocketOpen	)
				}

				this.oWebSocket = new WebSocket(this.sUri)

				this.oWebSocket	.addEventListener("close"	,	this.onWebSocketClosed	)
				this.oWebSocket	.addEventListener("error"	,	this.onWebSocketError	)
				this.oWebSocket	.addEventListener("message"	,	this.onWebSocketMessage	)
				this.oWebSocket	.addEventListener("open"	,	this.onWebSocketOpen	)

			},

			addEventListener: function(sEvent, fHandler)
			{
				sEvent = this._eventName(sEvent)

				if(typeof sEvent !== "string" || sEvent === "" || this.oEventHandler[sEvent] === undefined)
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.addEventListener\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected [targetLanguageChanged] found \"" + sEvent + "\""
					)
				}
				if(typeof fHandler !== "function")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Translator.addEventListener\""
						+	"\nArgument: \"handler\""
						+	"\nType: expected \"function\" found \"" + typeof fHandler + "\""
						+	"\nValue: expected function found\"" + fHandler + "\""
					)
				}

				this.oEventHandler[sEvent].push(fHandler)

				return this
			},
			removeEventListener: function(sEvent, fHandler)
			{
				sEvent = this._eventName(sEvent)

				if(typeof sEvent !== "string" || sEvent === "" || this.oEventHandler[sEvent] === undefined)
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Connector.addEventListener\""
						+	"\nArgument: \"event\""
						+	"\nType: expected \"string\" found \"" + typeof sEvent + "\""
						+	"\nValue: expected [targetLanguageChanged] found \"" + sEvent + "\""
					)
				}
				if(typeof fHandler !== "function")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Connector.addEventListener\""
						+	"\nArgument: \"handler\""
						+	"\nType: expected \"function\" found \"" + typeof fHandler + "\""
						+	"\nValue: expected function found\"" + fHandler + "\""
					)
				}

				this.oEventHandler[sEvent].splice(this.oEventHandler[sEvent].indexof(fHandler), 1)

				return this
			},
			send: function(sType, vValue)
			{
				if(typeof sType !== "string" || sType === "")
				{
					throw new TypeError
					(
							"incorrect type or unusable value"
						+	"\nMethod: \"Connector.send\""
						+	"\nArgument: \"language\""
						+	"\nType: expected \"string\" found \"" + typeof sType + "\""
						+	"\nValue: expected !\"\" found \"" + sType + "\""
					)
				}

				if(this.oWebSocket.readyState === WebSocket.OPEN)
				{
					switch(sType)
					{
						case "language":
							this.oWebSocket.send(JSON.stringify({type: "language", value: vValue || Translator.sourceLanguage()}))
							break
						case "message":
							this.oWebSocket.send(JSON.stringify({type: "message", value: vValue}))
							break
					}
				}

				return this
			},
		}

		document.addEventListener
		(
			"DOMContentLoaded",
			function()
			{
				oConnector.initialize()
			}
		)
		window.Connector = oConnector
	}()
)
